==> component-preannouncement-1.2.10-20190527.064020-14-sources.jar: com/bxm/newidea/cloud/ribbon/EnvDiscoveryEnabledRule.java <==
package com.bxm.newidea.cloud.ribbon;

import com.bxm.newidea.cloud.constant.EnvConstant;
import com.google.common.base.Optional;
import com.google.common.collect.Lists;
import com.netflix.loadbalancer.ILoadBalancer;
import com.netflix.loadbalancer.Server;
import com.netflix.loadbalancer.ZoneAvoidanceRule;
import com.netflix.niws.loadbalancer.DiscoveryEnabledServer;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 不同环境的负载规则分发
 * @author liujia
 */
public class EnvDiscoveryEnabledRule extends ZoneAvoidanceRule {

    private static final Logger LOGGER = LoggerFactory.getLogger(EnvDiscoveryEnabledRule.class);

    /**
     * 当前配置是否预发环境
     */
    private boolean preEnv;

    public EnvDiscoveryEnabledRule() {
        super();
    }

    public void setPreEnv(boolean preEnv) {
        this.preEnv = preEnv;
    }

    @Override
    public Server choose(Object key) {
        ILoadBalancer lb = getLoadBalancer();
        final List<Server> allServers = lb.getAllServers();
        // 匹配成功的server
        final List<Server> matchedMetaServers = new ArrayList<>();
        //未匹配的服务
        List<Server> unmatchServers = Lists.newArrayList();

        final String ribbonVal;
        //如果启用了预发配置，直接使用预发逻辑。
        if (this.preEnv) {
            ribbonVal = EnvConstant.ENV_HEADER_KEY;
        } else {
            final RibbonFilterContext context = RibbonFilterContextHolder.getCurrentContext();
            LOGGER.debug("contenxt:{}", context);
            final Map<String, String> attributes = context.getAttributes();
            // 取得接口端传入的参数
            ribbonVal = attributes.get(EnvConstant.ENV_HEADER_KEY);
        }

        LOGGER.debug("request server:{},ribbonVal:{},preEnv:{}", key, ribbonVal, this.preEnv);

        if (StringUtils.isNotBlank(ribbonVal)) {
            for (Server server : allServers) {
                if (server instanceof DiscoveryEnabledServer) {
                    final DiscoveryEnabledServer discoveryEnabledServer = (DiscoveryEnabledServer) server;
                    final Map<String, String> metadata = discoveryEnabledServer.getInstanceInfo().getMetadata();
                    final String env = metadata.get(EnvConstant.ENV_HEADER_KEY);
                    // 获取符合条件的服务
                    if (StringUtils.isNotBlank(env) && EnvConstant.PRE_ENV_VALUE.equals(env)) {
                        matchedMetaServers.add(server);
                    } else {
                        unmatchServers.add(server);
                    }
                }
            }

            LOGGER.info("key:{},allserver:{},matchServers:{},unmatchServer:{},ribbonVal:{}", key, allServers.size(),
                    matchedMetaServers.size(), unmatchServers.size(), ribbonVal);

            //优先走自定义路由。即满足灰度要求的server
            if (!matchedMetaServers.isEmpty()) {
                Optional<Server> server = getPredicate().chooseRoundRobinAfterFiltering(matchedMetaServers, key);
                if (server.isPresent()) {
                    LOGGER.debug("choose server: " + server.get().getId() + ",env:" + ribbonVal + ",key:" + key);
                    return server.get();
                } else {
                    LOGGER.warn("请求头信息中包含预发键值，但是未找到符合条件的服务，请求key：" + key);
                    return null;
                }
            } else {
                LOGGER.warn("未命中预发服务，直接使用全部服务：" + key);
                unmatchServers = allServers;
            }
            LOGGER.debug("请求头信息中包含预发键值，但是未找到对应配置的服务，请检查服务部署的eureka metadata，请求key：{},ribbonVal:{}"
                    , key
                    , ribbonVal);
        } else {
            unmatchServers = allServers;
        }

        //不存在预发键值，进行默认的路由
        Optional<Server> server = getPredicate().chooseRoundRobinAfterFiltering(unmatchServers, key);
        if (server.isPresent()) {
            LOGGER.debug("choose server: " + server.get().getId() + ",key:" + key + ",ribbonVal:" + ribbonVal);
            return server.get();
        } else {
            return null;
        }
    }
}

==> component-preannouncement-1.2.11-20190707.130328-12-sources.jar: com/bxm/newidea/cloud/ribbon/EnvDiscoveryEnabledRule.java <==
package com.bxm.newidea.cloud.ribbon;

import com.bxm.newidea.cloud.constant.EnvConstant;
import com.google.common.base.Optional;
import com.google.common.collect.Lists;
import com.netflix.loadbalancer.ILoadBalancer;
import com.netflix.loadbalancer.Server;
import com.netflix.loadbalancer.ZoneAvoidanceRule;
import com.netflix.niws.loadbalancer.DiscoveryEnabledServer;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 不同环境的负载规则分发
 * @author liujia
 */
public class EnvDiscoveryEnabledRule extends ZoneAvoidanceRule {

    private static final Logger LOGGER = LoggerFactory.getLogger(EnvDiscoveryEnabledRule.class);

    /**
     * 当前配置是否预发环境
     */
    private boolean preEnv;

    public EnvDiscoveryEnabledRule() {
        super();
    }

    public void setPreEnv(boolean preEnv) {
        this.preEnv = preEnv;
    }

    @Override
    public Server choose(Object key) {
        ILoadBalancer lb = getLoadBalancer();
        final List<Server> allServers = lb.getAllServers();
        // 匹配成功的server
        final List<Server> matchedMetaServers = new ArrayList<>();
        //未匹配的服务
        List<Server> unmatchServers = Lists.newArrayList();

        final String ribbonVal;
        //如果启用了预发配置，直接使用预发逻HtmlContentUtils辑。
        if (this.preEnv) {
            ribbonVal = EnvConstant.ENV_HEADER_KEY;
        } else {
            final RibbonFilterContext context = RibbonFilterContextHolder.getCurrentContext();
            LOGGER.debug("contenxt:{}", context);
            final Map<String, String> attributes = context.getAttributes();
            // 取得接口端传入的参数
            ribbonVal = attributes.get(EnvConstant.ENV_HEADER_KEY);
        }

        for (Server server : allServers) {
            if (server instanceof DiscoveryEnabledServer) {
                final DiscoveryEnabledServer discoveryEnabledServer = (DiscoveryEnabledServer) server;
                final Map<String, String> metadata = discoveryEnabledServer.getInstanceInfo().getMetadata();
                final String env = metadata.get(EnvConstant.ENV_HEADER_KEY);
                // 获取符合条件的服务
                if (StringUtils.isNotBlank(env) && EnvConstant.PRE_ENV_VALUE.equals(env)) {
                    matchedMetaServers.add(server);
                } else {
                    unmatchServers.add(server);
                }
            }
        }

        LOGGER.info("key:{},allserver:{},matchServers:{},unmatchServer:{},ribbonVal:{},preEnv:{}", key, allServers.size(),
                matchedMetaServers.size(), unmatchServers.size(), ribbonVal, this.preEnv);

        if (StringUtils.isNotBlank(ribbonVal)) {
            //优先走自定义路由。即满足灰度要求的server
            if (!matchedMetaServers.isEmpty()) {
                Optional<Server> server = getPredicate().chooseRoundRobinAfterFiltering(matchedMetaServers, key);
                if (server.isPresent()) {
                    LOGGER.debug("choose server: " + server.get().getId() + ",env:" + ribbonVal + ",key:" + key);
                    return server.get();
                } else {
                    LOGGER.warn("请求头信息中包含预发键值，但是未找到符合条件的服务，请求key：" + key);
                    return null;
                }
            } else {
                LOGGER.warn("未命中预发服务，直接使用全部服务：" + key);
                unmatchServers = allServers;
            }
            LOGGER.debug("请求头信息中包含预发键值，但是未找到对应配置的服务，请检查服务部署的eureka metadata，请求key：{},ribbonVal:{}"
                    , key
                    , ribbonVal);
        } else if (unmatchServers.size() == 0) {
            unmatchServers = allServers;
        }

        //不存在预发键值，进行默认的路由
        Optional<Server> server = getPredicate().chooseRoundRobinAfterFiltering(unmatchServers, key);
        if (server.isPresent()) {
            LOGGER.debug("choose server: " + server.get().getId() + ",key:" + key + ",ribbonVal:" + ribbonVal);
            return server.get();
        } else {
            return null;
        }
    }
}
